/*****************************************************************************
 * audioscrobbler.c: VLM service-discovery plugin
 *****************************************************************************
 * Copyright (C) 2000-2006 the VideoLAN team
 * $Id $
 *
 * Authors: Kenneth Ostby <kenneo@idi.ntnu.no>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

/*****************************************************************************
 * Preamble
 *****************************************************************************/
#include <stdlib.h>                                      /* malloc(), free() */

#include <vlc/vlc.h>
#include <vlc/intf.h>
#include <vlc_interaction.h>
#include <vlc_playlist.h>

#include <vlc/input.h>

#include "network.h"

#include <errno.h>

#ifdef HAVE_SYS_TIME_H
#    include <sys/time.h>
#endif

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif



/*****************************************************************************
 * Module descriptor
 *****************************************************************************/
static int  Open(vlc_object_t *p_this);
static void Close(vlc_object_t *p_this);
static void changePassword(vlc_object_t *,char const *,vlc_value_t,vlc_value_t,void *);
static void changeUsername(vlc_object_t *,char const *,vlc_value_t,vlc_value_t,void *);

#define REQUEST_HOST		"post.audioscrobbler.com"
#define REQUEST_GET            "GET /?hs=true&p=1.1&c=%s&v=%s&u=%s HTTP/1.1\r\nHost: post.audioscrobbler.com\r\n\r\n"
#define REQUEST_PORT            80
#define CLIENT_NAME		"tst"
#define CLIENT_VERSION		"1.0"
#define CLIENT_USER		"test"
#define USERNAME_TEXT		"Username"
#define USERNAME_LONGTEXT	"Your username for the audioscrobbler network"
#define PASSWORD_TEXT		"Password"
#define PASSWORD_LONGTEXT	"Your username for the audioscrobbler network"

vlc_module_begin();
    set_shortname( "audioscrobbler" );
    set_category( CAT_INTERFACE );
    add_string( "audios-user", "", changeUsername, USERNAME_TEXT,
                 USERNAME_LONGTEXT, VLC_FALSE );
    add_string( "audios-password", "", changePassword, PASSWORD_TEXT,
                 PASSWORD_LONGTEXT, VLC_FALSE );
    set_description( _("VLC Audioscrobbler plugin") );
    set_capability( "services_discovery", 0 );
    set_callbacks( Open , Close );
vlc_module_end();

/*****************************************************************************
 * Local prototypes.
 *****************************************************************************/
static void Run(services_discovery_t *);
static int handshake(services_discovery_t *p_sd);

typedef struct 
{
	int socket_fd;
}scrobbler_conn_t;

struct services_discovery_sys_t
{
    char *ps_challenge;
    unsigned int ui_challengeSize;
    char *psz_username;
    char *psz_password;
    char *psz_posturl;
    scrobbler_conn_t *p_scrob_con;
}; 

/*****************************************************************************
 * Init audioscrobbler connection
 *****************************************************************************/
static int handshake(services_discovery_t *p_sd){
	services_discovery_sys_t *p_sys;
        if(!p_sd) return -1;
	if(!(p_sys = (services_discovery_sys_t*)p_sd->p_sys)) return -2; 

        if(!p_sys->p_scrob_con){
                if(!(p_sys->p_scrob_con = (scrobbler_conn_t*)malloc(sizeof(scrobbler_conn_t)))){
			msg_Info(p_sd,"No memory to allocate");
		}
        }

       if( (p_sys->p_scrob_con->socket_fd = net_ConnectTCP(p_sd,REQUEST_HOST,REQUEST_PORT)) < 0 ){
                return -3;
       }
       /*
	* Send cool message 
	*/
       net_Printf(p_sd,
		    p_sys->p_scrob_con->socket_fd,
		    NULL,
		    REQUEST_GET,
		    CLIENT_NAME,
		    CLIENT_VERSION,
		    p_sys->psz_username);

       uint8_t *buffer = (uint8_t*)malloc(1024);
       int read = 0; 
       bzero(buffer,1024);

       /*
	* Read cool message 
	*/
       read = net_Read(p_sd,
		p_sys->p_scrob_con->socket_fd,
		NULL,
		buffer,
		1024,
		VLC_FALSE);
		    
       net_Close(p_sys->p_scrob_con->socket_fd);
       msg_Info(p_sd,buffer);

       uint8_t *body = NULL;
       /**
	* Get the content..
	*/
	/*
       while(*(buffer++)){
	       if(*buffer == '\r' && 
		  *(buffer+1) == '\n' &&
		  *(buffer+2) == '\r' &&	       
		  *(buffer+3) == '\n'){
		body = buffer+4;
		break;
	       }
       }*/

       msg_Info(p_sd,"Body:");
       //msg_Info(p_sd,body);

       if(buffer) free(buffer);
    return 1;
}

/*****************************************************************************
 * Open: initialize dummy interface
 *****************************************************************************/
static int Open(vlc_object_t *p_this ){
    msg_Info(p_this,"Loaded the module");
    
    services_discovery_t *p_sd = (services_discovery_t*) p_this;
    services_discovery_sys_t *p_audio = 
	    (services_discovery_sys_t*)malloc(sizeof(services_discovery_sys_t));

    p_audio->ps_challenge = NULL;
    p_audio->psz_username = NULL;
    p_audio->psz_password = NULL;

    /*
     * TODO: Debug 
     * Should be set by callbacks.
     * */
    p_audio->psz_username = CLIENT_USER;


    p_sd->p_sys = p_audio;

    msg_Info(p_this,"Loaded the module");

    int response = 0; 
    response = handshake(p_sd);
    
    msg_Info(p_this, "Resp: %d from handshake",response);
    
    p_sd->pf_run = Run;
    return VLC_SUCCESS;
}

/*****************************************************************************
 * Close:
 *****************************************************************************/
static void Close( vlc_object_t *p_this )
{
    msg_Info(p_this,"Exiting the module");
    services_discovery_sys_t *p_sd = (services_discovery_sys_t*)p_this;

    // Close the audioscrobbler connection.

    //free(p_sd->p_sys);
}

/*****************************************************************************
 * Run: main loop
 *****************************************************************************/
static void Run( services_discovery_t *p_sd )
{
    msg_Info(p_sd,"Startup");
    while(!p_sd->b_die){
	    /*
	     * if (current_song.mins_into > 150) && current_song != prev_song 
	     * then 
	     *  update audioscrobbler with new information
	     * endif
	     */
    }
    msg_Info(p_sd,"Shutdown");
}


/*****************************************************************************
 * Change the password
 *****************************************************************************/
static void changePassword(vlc_object_t *p_this,char const *name,vlc_value_t value0,vlc_value_t value1,void *pointer)
{
    msg_Info(p_this,"Password changed");
}

/*****************************************************************************
 * Change the username
 *****************************************************************************/
static void changeUsername(vlc_object_t *p_this,char const *name,vlc_value_t value0,vlc_value_t value1,void *pointer)
{
    msg_Info(p_this,"Username change");
}
